@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -27,10 +27,8 @@ import re
 import tempfile
 from io import BytesIO
 from pathlib import Path
-from typing import Any, BinaryIO
-
-# External imports
-import PIL.Image
+from typing import Any, BinaryIO, Literal
+from urllib.parse import quote
 
 # Bokeh imports
 from ...util.serialization import convert_datetime_type
@@ -51,6 +49,7 @@ from .string import Regex
 #-----------------------------------------------------------------------------
 
 __all__ = (
+    'CSSLength',
     'DashPattern',
     'FontSize',
     'HatchPatternType',
@@ -77,7 +76,7 @@ class DashPattern(Either):
     To indicate that dashing is turned off (solid lines), specify the empty
     list [].
 
-    .. _HTML5 Canvas dash specification style: http://www.w3.org/html/wg/drafts/2dcontext/html5_canvas/#dash-list
+    .. _HTML5 Canvas dash specification style: https://html.spec.whatwg.org/#dash-list
 
     """
 
@@ -103,13 +102,22 @@ class DashPattern(Either):
             try:
                 return self._dash_patterns[value]
             except KeyError:
-                return [int(x) for x in  value.split()]
+                return [int(x) for x in value.split()]
         else:
             return value
 
-class FontSize(String):
+CSS_LENGTH_RE = re.compile(r"^[0-9]+(.[0-9]+)?(%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px)$", re.I)
 
-    _font_size_re = re.compile(r"^[0-9]+(.[0-9]+)?(%|em|ex|ch|ic|rem|vw|vh|vi|vb|vmin|vmax|cm|mm|q|in|pc|pt|px)$", re.I)
+class CSSLength(String):
+
+    def validate(self, value: Any, detail: bool = True) -> None:
+        super().validate(value, detail)
+
+        if not (isinstance(value, str) and CSS_LENGTH_RE.match(value)):
+            msg = "" if not detail else f"{value!r} is not a valid CSS length"
+            raise ValueError(msg)
+
+class FontSize(String):
 
     def validate(self, value: Any, detail: bool = True) -> None:
         super().validate(value, detail)
@@ -118,7 +126,7 @@ class FontSize(String):
             if len(value) == 0:
                 msg = "" if not detail else "empty string is not a valid font size value"
                 raise ValueError(msg)
-            elif not self._font_size_re.match(value):
+            elif not CSS_LENGTH_RE.match(value):
                 msg = "" if not detail else f"{value!r} is not a valid font size value"
                 raise ValueError(msg)
 
@@ -137,8 +145,8 @@ class HatchPatternType(Either):
     def __str__(self) -> str:
         return self.__class__.__name__
 
-class Image(Property):
-    """ Accept image file types, e.g PNG, JPEG, TIFF, etc.
+class Image(Property[str]):
+    """ Accept image file types, e.g. PNG, JPEG, TIFF, etc.
 
     This property can be configured with:
 
@@ -154,8 +162,9 @@ class Image(Property):
 
     def validate(self, value: Any, detail: bool = True) -> None:
         import numpy as np
+        import PIL.Image
 
-        if isinstance(value, (str, Path, PIL.Image.Image)):
+        if isinstance(value, str | Path | PIL.Image.Image):
             return
 
         if isinstance(value, np.ndarray):
@@ -165,24 +174,39 @@ class Image(Property):
         msg = "" if not detail else f"invalid value: {value!r}; allowed values are string filenames, PIL.Image.Image instances, or RGB(A) NumPy arrays"
         raise ValueError(msg)
 
-    def transform(self, value):
+    def transform(self, value: Any) -> str:
         import numpy as np
+        import PIL.Image
+
         if isinstance(value, np.ndarray):
             value = PIL.Image.fromarray(value)
 
         if isinstance(value, str):
-            return value
+            if value.startswith(("data:", "http://", "https://", "file://")):
+                return value
+
+            path = Path(value)
+            if path.exists():
+                value = path
+            else:
+                return value
+
+        def data_image(format: str, encoding: Literal["utf8", "base64"], data: str) -> str:
+            return f"data:image/{format};{encoding},{data}"
+
+        if isinstance(value, Path) and value.suffix == ".svg":
+            return data_image("svg+xml", "utf8", quote(value.read_text()))
 
         # tempfile doesn't implement IO interface (https://bugs.python.org/issue33762)
-        if isinstance(value, (Path, BinaryIO, tempfile._TemporaryFileWrapper)):
+        if isinstance(value, Path | BinaryIO | tempfile._TemporaryFileWrapper):
             value = PIL.Image.open(value)
 
         if isinstance(value, PIL.Image.Image):
             out = BytesIO()
-            fmt = value.format or "PNG"
-            value.save(out, fmt)
+            format = value.format or "PNG"
+            value.save(out, format)
             encoded = base64.b64encode(out.getvalue()).decode('ascii')
-            return f"data:image/{fmt.lower()};base64,{encoded}"
+            return data_image(format.lower(), "base64", encoded)
 
         raise ValueError(f"Could not transform {value!r}")
 
