@@ -39,6 +39,7 @@ from ..core.enums import (
     Place,
     PlaceType,
     ResetPolicy,
+    WindowAxis,
 )
 from ..core.properties import (
     Bool,
@@ -60,7 +61,7 @@ from ..core.properties import (
     Tuple,
 )
 from ..core.property.struct import Optional
-from ..core.property_mixins import ScalarFillProps, ScalarLineProps
+from ..core.property_mixins import ScalarFillProps, ScalarHatchProps, ScalarLineProps
 from ..core.query import find
 from ..core.validation import error, warning
 from ..core.validation.errors import (
@@ -119,7 +120,7 @@ class Plot(LayoutDOM):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     def select(self, *args, **kwargs):
@@ -205,10 +206,10 @@ class Plot(LayoutDOM):
         '''
         return self in gridplot.column(col)
 
-    def _axis(self, *sides):
-        objs = []
-        for s in sides:
-            objs.extend(getattr(self, s, []))
+    def _axis(self, *sides: PlaceType):
+        objs: list[Model] = []
+        for side in sides:
+            objs.extend(getattr(self, side, []))
         axis = [obj for obj in objs if isinstance(obj, Axis)]
         return _list_attr_splat(axis)
 
@@ -284,7 +285,12 @@ class Plot(LayoutDOM):
         self.toolbar.tools = tools
 
     def add_layout(self, obj: Renderer, place: PlaceType = "center") -> None:
-        ''' Adds an object to the plot in a specified place.
+        ''' Adds an object to the plot in the specified place.
+
+        If the renderer is already a part of a plot, this operation will move
+        it to the new location. If you need finer control than this, you can
+        manipulate ``left``, ``right``, ``above``, ``below`` or ``center``
+        Plot's properties manually.
 
         Args:
             obj (Renderer) : the object to add to the Plot
@@ -300,6 +306,11 @@ class Plot(LayoutDOM):
                 f"Invalid place '{place}' specified. Valid place values are: {nice_join(Place)}",
             )
 
+        for name in Place:
+            panel = getattr(self, name)
+            while obj in panel:
+                panel.remove(obj)
+
         getattr(self, place).append(obj)
 
     def add_tools(self, *tools: Tool | str) -> None:
@@ -449,7 +460,7 @@ class Plot(LayoutDOM):
         if not self.x_range: missing.append('x_range')
         if not self.y_range: missing.append('y_range')
         if missing:
-            return ", ".join(missing) + " [%s]" % self
+            return ", ".join(missing) + f" [{self}]"
 
     @error(REQUIRED_SCALE)
     def _check_required_scale(self) -> str | None:
@@ -457,7 +468,7 @@ class Plot(LayoutDOM):
         if not self.x_scale: missing.append('x_scale')
         if not self.y_scale: missing.append('y_scale')
         if missing:
-            return ", ".join(missing) + " [%s]" % self
+            return ", ".join(missing) + f" [{self}]"
 
     @error(INCOMPATIBLE_SCALE_AND_RANGE)
     def _check_compatible_scale_and_ranges(self) -> str | None:
@@ -469,20 +480,20 @@ class Plot(LayoutDOM):
 
         if self.x_scale is not None:
             for rng in x_ranges:
-                if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.x_scale, (LinearScale, LogScale)):
+                if isinstance(rng, DataRange1d | Range1d) and not isinstance(self.x_scale, LinearScale | LogScale):
                     incompatible.append(f"incompatibility on x-dimension: {rng}, {self.x_scale}")
                 elif isinstance(rng, FactorRange) and not isinstance(self.x_scale, CategoricalScale):
                     incompatible.append(f"incompatibility on x-dimension: {rng}, {self.x_scale}")
 
         if self.y_scale is not None:
             for rng in y_ranges:
-                if isinstance(rng, (DataRange1d, Range1d)) and not isinstance(self.y_scale, (LinearScale, LogScale)):
+                if isinstance(rng, DataRange1d | Range1d) and not isinstance(self.y_scale, LinearScale | LogScale):
                     incompatible.append(f"incompatibility on y-dimension: {rng}, {self.y_scale}")
                 elif isinstance(rng, FactorRange) and not isinstance(self.y_scale, CategoricalScale):
                     incompatible.append(f"incompatibility on y-dimension: {rng}, {self.y_scale}")
 
         if incompatible:
-            return ", ".join(incompatible) + " [%s]" % self
+            return ", ".join(incompatible) + f" [{self}]"
 
     @warning(MISSING_RENDERERS)
     def _check_missing_renderers(self) -> str | None:
@@ -516,17 +527,6 @@ class Plot(LayoutDOM):
     The (default) data range of the vertical dimension of the plot.
     """)
 
-    @classmethod
-    def _scale(cls, scale: Literal["auto", "linear", "log", "categorical"]) -> Scale:
-        if scale in ["auto", "linear"]:
-            return LinearScale()
-        elif scale == "log":
-            return LogScale()
-        if scale == "categorical":
-            return CategoricalScale()
-        else:
-            raise ValueError(f"Unknown mapper_type: {scale}")
-
     x_scale = Instance(Scale, default=InstanceDefault(LinearScale), help="""
     What kind of scale to use to convert x-coordinates in data space
     into x-coordinates in screen space.
@@ -565,6 +565,17 @@ class Plot(LayoutDOM):
     .. note:: This feature is experimental and may change in the short term.
     """)
 
+    window_axis = Enum(WindowAxis, default="none", help="""
+    An axis to use for windowed auto-ranging when there are data ranges
+    present on the plot. For example, if ``window_axis`` is set to the
+    value ``"x"`` then any data ranges in the y-dimension will compute their
+    auto-ranged extents using only data inside the range bounds for the
+    x-axis as configured in the current viewport.
+
+    If set to "none" (the default) then auto-ranging will use all available
+    data, regardless of viewport.
+    """)
+
     hidpi = Bool(default=True, help="""
     Whether to use HiDPI mode when available.
     """)
@@ -649,7 +660,7 @@ class Plot(LayoutDOM):
     Allows to specify which frame edges to align in multiple-plot layouts.
 
     The default is to align all edges, but users can opt-out from alignment
-    of each individual edge or all edges. Note also that other proproperties
+    of each individual edge or all edges. Note also that other properties
     may disable alignment of certain edges, especially when using fixed frame
     size (``frame_width`` and ``frame_height`` properties).
     """)
@@ -698,13 +709,21 @@ class Plot(LayoutDOM):
 
     """)
 
-    background_props = Include(ScalarFillProps, prefix="background", help="""
+    background_fill_props = Include(ScalarFillProps, prefix="background", help="""
+    The {prop} for the plot background style.
+    """)
+
+    background_hatch_props = Include(ScalarHatchProps, prefix="background", help="""
     The {prop} for the plot background style.
     """)
 
     background_fill_color = Override(default='#ffffff')
 
-    border_props = Include(ScalarFillProps, prefix="border", help="""
+    border_fill_props = Include(ScalarFillProps, prefix="border", help="""
+    The {prop} for the plot border style.
+    """)
+
+    border_hatch_props = Include(ScalarHatchProps, prefix="border", help="""
     The {prop} for the plot border style.
     """)
 
@@ -757,25 +776,46 @@ class Plot(LayoutDOM):
     """)
 
     lod_factor = Int(10, help="""
-    Decimation factor to use when applying level-of-detail decimation.
+    Decimation factor to use when applying level-of-detail mode.
+
+    A ``lod_factor`` of N means that only every Nth point in the data source
+    will be drawn while interactive events are active. For example, if
+    ``lod_factor=200`` then only every 200th point will be drawn.
+
+    The level-of-detail mode is intended to preserve interactive response
+    times on HTML canvas plots when there are a large number of data points.
+
+    Note that a possible alternative to level-of-detail mode is using the
+    WebGL ``output_backend``. WebGL rendering may allow very large data sets
+    to remain interactive without any level-of-detail downsampling. When
+    WebGL output is enabled, level-of-detail mode is not used.
     """)
 
     lod_threshold = Nullable(Int, default=2000, help="""
     A number of data points, above which level-of-detail downsampling may
-    be performed by glyph renderers. Set to ``None`` to disable any
-    level-of-detail downsampling.
+    be performed by glyph renderers. For example, if ``lod_threshold=10000``
+    then level-of-detail mode will not be activated if there are fewer than
+    10000 points in the data source.
+
+    Set to ``None`` to disable any level-of-detail downsampling at all.
     """)
 
     lod_interval = Int(300, help="""
     Interval (in ms) during which an interactive tool event will enable
     level-of-detail downsampling.
+
+    If a plot needs to be re-drawn within ``lod_interval`` milliseconds
+    of the last interactive event starting, then level-of-detail mode will
+    be activated. Larger values mean the level-of-detail mode will be
+    "easier" to turn on.
     """)
 
     lod_timeout = Int(500, help="""
     Timeout (in ms) for checking whether interactive tool events are still
     occurring. Once level-of-detail mode is enabled, a check is made every
     ``lod_timeout`` ms. If no interactive tool events have happened,
-    level-of-detail mode is disabled.
+    level-of-detail mode is disabled. Larger values mean the level-of-detail
+    mode will be "slower" to turn off.
     """)
 
     output_backend = Enum(OutputBackend, default="canvas", help="""
@@ -823,7 +863,7 @@ class Plot(LayoutDOM):
     """)
 
     reset_policy = Enum(ResetPolicy, default="standard", help="""
-    How a plot should respond to being reset. By deafult, the standard actions
+    How a plot should respond to being reset. By default, the standard actions
     are to clear any tool state history, return plot ranges to their original
     values, undo all selections, and emit a ``Reset`` event. If customization
     is desired, this property may be set to ``"event_only"``, which will
@@ -868,13 +908,15 @@ class Plot(LayoutDOM):
         This feature is experimental and may change in the short term.
     """)
 
+    context_menu = Override(default="auto")
+
 class GridPlot(LayoutDOM, GridCommon):
     """ Collection of plots and other layoutables on arranged on a rectangular grid.
 
     """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     toolbar = Instance(Toolbar, default=InstanceDefault(Toolbar), help="""
@@ -891,7 +933,7 @@ class GridPlot(LayoutDOM, GridCommon):
         Tuple(Instance(LayoutDOM), Int, Int),
         Tuple(Instance(LayoutDOM), Int, Int, Int, Int)), default=[], help="""
     A list of subplots with their associated position in the grid, row and column
-    index and optional row and column spans (the defaul span is 1).
+    index and optional row and column spans (the default span is 1).
     """)
 
     @error(REPEATED_LAYOUT_CHILD)
@@ -920,7 +962,7 @@ class _list_attr_splat(list):
         if attr in dir(list):
             return list.__getattribute__(self, attr)
         if len(self) == 0:
-            raise AttributeError("Trying to access %r attribute on an empty 'splattable' list" % attr)
+            raise AttributeError(f"Trying to access {attr!r} attribute on an empty 'splattable' list")
         if len(self) == 1:
             return getattr(self[0], attr)
         try:
