@@ -41,9 +41,8 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import difflib
-import typing as tp
 from math import nan
-from typing import Literal
+from typing import Any, Callable, ClassVar
 
 # Bokeh imports
 from ..core.enums import (
@@ -51,16 +50,16 @@ from ..core.enums import (
     Dimension,
     Dimensions,
     KeyModifierType,
+    PanDirection,
     RegionSelectionMode,
     SelectionMode,
-    ToolIcon,
+    ToolName,
     TooltipAttachment,
     TooltipFieldFormatter,
 )
 from ..core.has_props import abstract
 from ..core.properties import (
     Alpha,
-    Any,
     AnyRef,
     Auto,
     Bool,
@@ -72,7 +71,6 @@ from ..core.properties import (
     Either,
     Enum,
     Float,
-    Image,
     Instance,
     InstanceDefault,
     Int,
@@ -82,7 +80,7 @@ from ..core.properties import (
     Nullable,
     Override,
     Percent,
-    Regex,
+    Required,
     Seq,
     String,
     Struct,
@@ -90,6 +88,7 @@ from ..core.properties import (
     TypeOfAttr,
 )
 from ..core.property.struct import Optional
+from ..core.property_aliases import IconLike
 from ..core.validation import error
 from ..core.validation.errors import NO_RANGE_TOOL_RANGES
 from ..model import Model
@@ -112,7 +111,7 @@ from .misc.group_by import GroupBy, GroupByModels, GroupByName
 from .nodes import Node
 from .ranges import Range
 from .renderers import DataRenderer, GlyphRenderer
-from .ui import UIElement
+from .ui import Menu, UIElement
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -123,6 +122,7 @@ __all__ = (
     'BoxEditTool',
     'BoxSelectTool',
     'BoxZoomTool',
+    'ClickPanTool',
     'CopyTool',
     'CrosshairTool',
     'CustomAction',
@@ -151,6 +151,7 @@ __all__ = (
     'Tap',
     'TapTool',
     'Tool',
+    'ToolMenu',
     'ToolProxy',
     'Toolbar',
     'UndoTool',
@@ -185,7 +186,7 @@ def _parse_modifiers(value: str) -> dict[KeyModifierType, bool]:
 
 def GlyphRendererOf(*types: type[Model]):
     """ Constraints ``GlyphRenderer.glyph`` to the given type or types. """
-    return TypeOfAttr(Instance(GlyphRenderer), "glyph", Either(*(Instance(tp) for tp in types)))
+    return TypeOfAttr(Instance(GlyphRenderer), "glyph", Either(*(Instance(type) for type in types)))
 
 @abstract
 class Tool(Model):
@@ -194,11 +195,10 @@ class Tool(Model):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    #Image has to be first! see #12775, temporary fix
-    icon = Nullable(Either(Image, Enum(ToolIcon), Regex(r"^\.")), help="""
+    icon = Nullable(IconLike, help="""
     An icon to display in the toolbar.
 
     The icon can provided as well known tool icon name, a CSS class selector,
@@ -217,7 +217,15 @@ class Tool(Model):
     Whether a tool button associated with this tool should appear in the toolbar.
     """)
 
-    _known_aliases: tp.ClassVar[dict[str, tp.Callable[[], Tool]]] = {}
+    group = Either(String, Bool, default=True, help="""
+    The name of the group this tool belongs to.
+
+    By default set to ``True``, indicating the default group. If set to
+    ``False``, it will prevent the tool from being grouped altogether
+    (regardless of ``Toolbar.group`` setting).
+    """)
+
+    _known_aliases: ClassVar[dict[str, Callable[[], Tool]]] = {}
 
     @classmethod
     def from_string(cls, name: str) -> Tool:
@@ -233,13 +241,13 @@ class Tool(Model):
             raise ValueError(f"unexpected tool name '{name}', {text} tools are {nice_join(matches)}")
 
     @classmethod
-    def register_alias(cls, name: str, constructor: tp.Callable[[], Tool]) -> None:
+    def register_alias(cls, name: str, constructor: Callable[[], Tool]) -> None:
         cls._known_aliases[name] = constructor
 
 class ToolProxy(Model):
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     tools = List(Instance(Tool))
@@ -253,7 +261,7 @@ class ActionTool(Tool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -263,7 +271,7 @@ class PlotActionTool(ActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -273,7 +281,7 @@ class GestureTool(Tool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -283,7 +291,7 @@ class Drag(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -293,7 +301,7 @@ class Scroll(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -303,7 +311,7 @@ class Tap(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -313,7 +321,7 @@ class SelectTool(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = Either(Auto, List(Instance(DataRenderer)), default="auto", help="""
@@ -328,7 +336,7 @@ class RegionSelectTool(SelectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     mode = Enum(RegionSelectionMode, default="replace", help="""
@@ -369,7 +377,7 @@ class InspectTool(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     toggleable = DeprecatedAlias("visible", since=(3, 4, 0))
@@ -380,9 +388,13 @@ class Toolbar(UIElement):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
+    tools = List(Either(Instance(Tool), Instance(ToolProxy)), help="""
+    A list of tools to add to the plot.
+    """)
+
     logo = Nullable(Enum("normal", "grey"), default="normal", help="""
     What version of the Bokeh logo to display on the toolbar. If
     set to None, no logo will be displayed.
@@ -393,29 +405,32 @@ class Toolbar(UIElement):
     If True, hides toolbar when cursor is not in canvas.
     """)
 
-    tools = List(Either(Instance(Tool), Instance(ToolProxy)), help="""
-    A list of tools to add to the plot.
+    group = Bool(default=True, help="""
+    Whether to group common tools.
     """)
 
-    active_drag: Literal["auto"] | Drag | ToolProxy | None = Either(Null, Auto, Instance(Drag), Instance(ToolProxy), default="auto", help="""
+    group_types = List(Enum(ToolName), default=["hover"], help="""
+    Only group tools of the given types.
+    """)
+
+    active_drag = Either(Null, Auto, Instance(Drag), Instance(ToolProxy), default="auto", help="""
     Specify a drag tool to be active when the plot is displayed.
     """)
 
-    active_inspect: Literal["auto"] | InspectTool | ToolProxy | tp.Sequence[InspectTool] | None = \
-        Either(Null, Auto, Instance(InspectTool), Instance(ToolProxy), Seq(Instance(InspectTool)), default="auto", help="""
+    active_inspect = Either(Null, Auto, Instance(InspectTool), Instance(ToolProxy), Seq(Instance(InspectTool)), default="auto", help="""
     Specify an inspection tool or sequence of inspection tools to be active when
     the plot is displayed.
     """)
 
-    active_scroll: Literal["auto"] | Scroll | ToolProxy | None = Either(Null, Auto, Instance(Scroll), Instance(ToolProxy), default="auto", help="""
+    active_scroll = Either(Null, Auto, Instance(Scroll), Instance(ToolProxy), default="auto", help="""
     Specify a scroll/pinch tool to be active when the plot is displayed.
     """)
 
-    active_tap: Literal["auto"] | Tap | ToolProxy | None = Either(Null, Auto, Instance(Tap), Instance(ToolProxy), default="auto", help="""
+    active_tap = Either(Null, Auto, Instance(Tap), Instance(ToolProxy), default="auto", help="""
     Specify a tap/click tool to be active when the plot is displayed.
     """)
 
-    active_multi: Literal["auto"] | GestureTool | ToolProxy | None = Either(Null, Auto, Instance(GestureTool), Instance(ToolProxy), default="auto", help="""
+    active_multi = Either(Null, Auto, Instance(GestureTool), Instance(ToolProxy), default="auto", help="""
     Specify an active multi-gesture tool, for instance an edit tool or a range
     tool.
 
@@ -425,6 +440,18 @@ class Toolbar(UIElement):
     be deactivated (i.e. the multi-gesture tool will take precedence).
     """)
 
+class ToolMenu(Menu):
+    """ Toolbar represented in a menu or context menu form.
+    """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    toolbar = Required(Instance(Toolbar), help="""
+    Reference to a toolbar.
+    """)
+
 class PanTool(Drag):
     ''' *toolbar icon*: |pan_icon|
 
@@ -442,7 +469,7 @@ class PanTool(Drag):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     dimensions = Enum(Dimensions, default="both", help="""
@@ -452,6 +479,26 @@ class PanTool(Drag):
     height of the plot.
     """)
 
+class ClickPanTool(PlotActionTool):
+    ''' A tool that allows to pan a plot by a fixed amount by clicking a button.
+
+    '''
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    direction = Required(Enum(PanDirection), help="""
+    The direction in which to pan the plot.
+
+    Accepted values are ``"left"``, ``"right"``, ``"up"`` or ``"down"``, or
+    their respective aliases ``"west"``, ``"east"``, ``"north"``, ``"south"``.
+    """)
+
+    factor = Percent(default=0.1, help="""
+    Percentage of the range to pan for each usage of the tool.
+    """)
+
 # TODO InstanceDefault() doesn't allow for lazy argument evaluation
 # DEFAULT_RANGE_OVERLAY = InstanceDefault(BoxAnnotation,
 DEFAULT_RANGE_OVERLAY = lambda: BoxAnnotation(
@@ -495,7 +542,7 @@ class RangeTool(Tool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     x_range = Nullable(Instance(Range), help="""
@@ -569,7 +616,7 @@ class WheelPanTool(Scroll):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     dimension = Enum(Dimension, default="width", help="""
@@ -624,7 +671,7 @@ class WheelZoomTool(Scroll):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     # ZoomBaseTool common {
@@ -761,13 +808,40 @@ class CustomAction(ActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
+    active = Bool(default=False, help="""
+    If ``True``, the tool is currently engaged for its activity.
+    """)
+
+    disabled = Bool(default=False, help="""
+    If ``True``, users can't interact with the tool in any way.
+    """)
+
     description = Override(default="Perform a Custom Action")
 
     callback = Nullable(Instance(Callback), help="""
     A Bokeh callback to execute when the custom action icon is activated.
+
+    This callback can return a boolean value to indicate the state of
+    the tool. This is only applicable if ``active_callback`` is ``None``.
+    """)
+
+    active_callback = Nullable(Either(Instance(Callback), Auto), default=None, help="""
+    A callback that allows to determine the state of the tool.
+
+    This callback is used to establish the initial and any subsequent state
+    of the tool. it must return a boolean value. A value of any other type
+    will be disregarded.
+
+    If ``"auto"`` value is used, then any click of the button will toggle
+    state. The initial state can be provided using ``active`` property.
+
+    If ``None`` value is used, then the tool isn't stateful.
+
+    .. note::
+        This property is experimental and may change at any point.
     """)
 
 class SaveTool(ActionTool):
@@ -787,7 +861,7 @@ class SaveTool(ActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     filename = Nullable(String, help="""
@@ -799,7 +873,7 @@ class SaveTool(ActionTool):
 class CopyTool(ActionTool):
     ''' *toolbar icon*: |copy_icon|
 
-    The copy tool is an action tool, that allows copying the rendererd contents of
+    The copy tool is an action tool, that allows copying the rendered contents of
     a plot or a collection of plots to system's clipboard. This tools is browser
     dependent and may not function in certain browsers, or require additional
     permissions to be granted to the web page.
@@ -810,7 +884,7 @@ class CopyTool(ActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class ResetTool(PlotActionTool):
@@ -827,7 +901,7 @@ class ResetTool(PlotActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class TapTool(Tap, SelectTool):
@@ -852,7 +926,7 @@ class TapTool(Tap, SelectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     mode = Enum(SelectionMode, default="toggle", help="""
@@ -949,7 +1023,7 @@ class CrosshairTool(InspectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     overlay = Either(
@@ -1054,10 +1128,10 @@ class BoxZoomTool(Drag):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    dimensions = Either(Enum(Dimensions), Auto, default="both", help="""
+    dimensions = Either(Enum(Dimensions), Auto, default="auto", help="""
     Which dimensions the zoom box is to be free in. By default, users may
     freely draw zoom boxes with any dimensions. If only "width" is supplied,
     the box will be constrained to span the entire vertical space of the plot,
@@ -1090,7 +1164,7 @@ class ZoomBaseTool(PlotActionTool):
     """ Abstract base class for zoom action tools. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = Either(Auto, List(Instance(DataRenderer)), default="auto", help="""
@@ -1129,7 +1203,7 @@ class ZoomInTool(ZoomBaseTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class ZoomOutTool(ZoomBaseTool):
@@ -1145,7 +1219,7 @@ class ZoomOutTool(ZoomBaseTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     maintain_focus = Bool(default=True, help="""
@@ -1173,7 +1247,7 @@ class BoxSelectTool(Drag, RegionSelectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     dimensions = Enum(Dimensions, default="both", help="""
@@ -1233,7 +1307,7 @@ class LassoSelectTool(Drag, RegionSelectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     overlay = Instance(PolyAnnotation, default=DEFAULT_POLY_OVERLAY, help="""
@@ -1267,7 +1341,7 @@ class PolySelectTool(Tap, RegionSelectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     overlay = Instance(PolyAnnotation, default=DEFAULT_POLY_OVERLAY, help="""
@@ -1327,7 +1401,7 @@ class CustomJSHover(Model):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     args = Dict(String, AnyRef, help="""
@@ -1434,7 +1508,7 @@ class HoverTool(InspectTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = Either(Auto, List(Instance(DataRenderer)), default="auto", help="""
@@ -1616,7 +1690,7 @@ class HelpTool(ActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     description = Override(default=DEFAULT_HELP_TIP)
@@ -1629,14 +1703,14 @@ class ExamineTool(ActionTool):
     ''' A tool that allows to inspect and configure a model. '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class FullscreenTool(ActionTool):
     ''' A tool that allows to enlarge a UI element to fullscreen. '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class UndoTool(PlotActionTool):
@@ -1651,7 +1725,7 @@ class UndoTool(PlotActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class RedoTool(PlotActionTool):
@@ -1666,7 +1740,7 @@ class RedoTool(PlotActionTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 @abstract
@@ -1676,10 +1750,10 @@ class EditTool(GestureTool):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    default_overrides = Dict(String, Any, default={}, help="""
+    default_overrides = Dict(String, AnyRef, default={}, help="""
     Padding values overriding ``ColumnarDataSource.default_values``.
 
     Defines values to insert into non-coordinate columns when a new glyph is
@@ -1709,7 +1783,7 @@ class PolyTool(EditTool):
     ''' A base class for polygon draw/edit tools. '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     vertex_renderer = Nullable(GlyphRendererOf(XYGlyph), help="""
@@ -1757,7 +1831,7 @@ class BoxEditTool(EditTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(LRTBGlyph, Rect, HStrip, VStrip), help="""
@@ -1817,7 +1891,7 @@ class PointDrawTool(EditTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(XYGlyph), help="""
@@ -1875,7 +1949,7 @@ class PolyDrawTool(PolyTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(MultiLine, Patches), help="""
@@ -1918,7 +1992,7 @@ class FreehandDrawTool(EditTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(MultiLine, Patches), help="""
@@ -1966,7 +2040,7 @@ class PolyEditTool(PolyTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(MultiLine, Patches), help="""
@@ -2000,7 +2074,7 @@ class LineEditTool(EditTool, Drag, Tap):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     renderers = List(GlyphRendererOf(Line), help="""
@@ -2029,17 +2103,32 @@ class LineEditTool(EditTool, Drag, Tap):
 Tool.register_alias("pan", lambda: PanTool(dimensions="both"))
 Tool.register_alias("xpan", lambda: PanTool(dimensions="width"))
 Tool.register_alias("ypan", lambda: PanTool(dimensions="height"))
+
+Tool.register_alias("pan_left", lambda: ClickPanTool(direction="left"))
+Tool.register_alias("pan_right", lambda: ClickPanTool(direction="right"))
+Tool.register_alias("pan_up", lambda: ClickPanTool(direction="up"))
+Tool.register_alias("pan_down", lambda: ClickPanTool(direction="down"))
+
+Tool.register_alias("pan_west", lambda: ClickPanTool(direction="west"))
+Tool.register_alias("pan_east", lambda: ClickPanTool(direction="east"))
+Tool.register_alias("pan_north", lambda: ClickPanTool(direction="north"))
+Tool.register_alias("pan_south", lambda: ClickPanTool(direction="south"))
+
 Tool.register_alias("xwheel_pan", lambda: WheelPanTool(dimension="width"))
 Tool.register_alias("ywheel_pan", lambda: WheelPanTool(dimension="height"))
+
 Tool.register_alias("wheel_zoom", lambda: WheelZoomTool(dimensions="both"))
 Tool.register_alias("xwheel_zoom", lambda: WheelZoomTool(dimensions="width"))
 Tool.register_alias("ywheel_zoom", lambda: WheelZoomTool(dimensions="height"))
+
 Tool.register_alias("zoom_in", lambda: ZoomInTool(dimensions="both"))
 Tool.register_alias("xzoom_in", lambda: ZoomInTool(dimensions="width"))
 Tool.register_alias("yzoom_in", lambda: ZoomInTool(dimensions="height"))
+
 Tool.register_alias("zoom_out", lambda: ZoomOutTool(dimensions="both"))
 Tool.register_alias("xzoom_out", lambda: ZoomOutTool(dimensions="width"))
 Tool.register_alias("yzoom_out", lambda: ZoomOutTool(dimensions="height"))
+
 Tool.register_alias("click", lambda: TapTool(behavior="inspect"))
 Tool.register_alias("tap", lambda: TapTool())
 Tool.register_alias("doubletap", lambda: TapTool(gesture="doubletap"))
