@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -22,10 +22,13 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from math import inf
+from typing import Any
 
 # Bokeh imports
 from ...core.has_props import abstract
 from ...core.properties import (
+    AnyRef,
+    Auto,
     Bool,
     Color,
     ColorHex,
@@ -48,6 +51,10 @@ from ...core.properties import (
     String,
     Tuple,
 )
+from ...core.property_aliases import IconLike
+from ...events import ModelEvent
+from ...util.deprecation import deprecated
+from ..dom import HTML
 from ..formatters import TickFormatter
 from ..ui import Tooltip
 from .widget import Widget
@@ -59,7 +66,7 @@ from .widget import Widget
 __all__ = (
     'AutocompleteInput',
     'Checkbox',
-    'ColorMap',
+    'PaletteSelect',
     'ColorPicker',
     'FileInput',
     'InputWidget',
@@ -72,13 +79,13 @@ __all__ = (
     'Switch',
     'TextAreaInput',
     'TextInput',
+    'ColorMap',
 )
 
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
 
-
 @abstract
 class InputWidget(Widget):
     ''' Abstract base class for input widgets.
@@ -86,10 +93,10 @@ class InputWidget(Widget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    title = String(default="", help="""
+    title = Either(String, Instance(HTML), default="", help="""
     Widget's label.
     """)
 
@@ -97,33 +104,37 @@ class InputWidget(Widget):
     Either a plain text or a tooltip with a rich HTML description of the function of this widget.
     """)
 
-    @classmethod
-    def coerce_value(cls, val):
-        prop_obj = cls.lookup('value')
-        if isinstance(prop_obj, Float):
-            return float(val)
-        elif isinstance(prop_obj, Int):
-            return int(val)
-        elif isinstance(prop_obj, String):
-            return str(val)
-        else:
-            return val
-
 #-----------------------------------------------------------------------------
 # General API
 #-----------------------------------------------------------------------------
 
+# TODO mark this as a one way event from server to client
+class ClearInput(ModelEvent):
+    """
+    Notifies the input widget that its input/value needs to be cleared.
+
+    This is specially useful for widgets whose value can't be simply cleared by
+    assigning to ``value`` (or equivalent) property.
+
+    """
+    event_name = "clear_input"
+
+    def __init__(self, model: InputWidget) -> None:
+        if not isinstance(model, InputWidget):
+            raise ValueError(f"{self.__class__.__name__} event only applies to input models, i.e. instances of bokeh.models.widgets.InputWidget")
+        super().__init__(model=model)
+
 class FileInput(InputWidget):
     ''' Present a file-chooser dialog to users and return the contents of the
     selected files.
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     value = Readonly(Either(String, List(String)), help='''
-    The base64-enconded contents of the file or files that were loaded.
+    The base64-encoded contents of the file or files that were loaded.
 
     If `multiple` is set to False (default), this value is a single string with the contents
     of the single file that was chosen.
@@ -197,6 +208,30 @@ class FileInput(InputWidget):
     selection of more than one file at a time should be possible.
     """)
 
+    directory = Bool(default=False, help="""
+    Whether to allow selection of directories instead of files.
+
+    The filename will be relative paths to the uploaded directory.
+
+    .. note::
+        When a directory is uploaded it will give add a confirmation pop up.
+        The confirmation pop up cannot be disabled, as this is a security feature
+        in the browser.
+
+    .. note::
+        The `accept` parameter only works with file extension.
+        When using `accept` with `directory`, the number of files
+        reported will be the total amount of files, not the filtered.
+    """)
+
+
+    def clear(self) -> None:
+        """ Clear the contents of this file input widget.
+
+        """
+        doc = self.document
+        if doc is not None:
+            doc.callbacks.send_event(ClearInput(self))
 
 class NumericInput(InputWidget):
     ''' Numeric input widget.
@@ -204,7 +239,7 @@ class NumericInput(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     value = Either(Null, Float, Int, help="""
@@ -243,7 +278,7 @@ class Spinner(NumericInput):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     value_throttled = Readonly(Either(Null, Float, Int), help="""
@@ -271,32 +306,36 @@ class ToggleInput(Widget):
     """ Base class for toggleable (boolean) input widgets. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     active = Bool(default=False, help="""
     The state of the widget.
     """)
 
+    label = String(default="", help="""
+    The label next to the input.
+    """)
+
 class Checkbox(ToggleInput):
     """ A checkbox widget. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    label = String(default="", help="""
-    The label next to the checkbox.
-    """)
-
 class Switch(ToggleInput):
     """ A checkbox-like widget. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    width = Override(default=32)
+    on_icon = Nullable(IconLike, default=None, help="""
+    """)
+
+    off_icon = Nullable(IconLike, default=None, help="""
+    """)
 
 class TextLikeInput(InputWidget):
     ''' Base class for text-like input widgets.
@@ -304,7 +343,7 @@ class TextLikeInput(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     value = String(default="", help="""
@@ -334,7 +373,7 @@ class TextInput(TextLikeInput):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     prefix = Nullable(String, help="""
@@ -353,7 +392,7 @@ class TextAreaInput(TextLikeInput):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     cols = Int(default=20, help="""
@@ -379,7 +418,7 @@ class PasswordInput(TextInput):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 
@@ -389,7 +428,7 @@ class AutocompleteInput(TextInput):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     completions = List(String, help="""
@@ -420,6 +459,10 @@ class AutocompleteInput(TextInput):
     match any substring of a completion string.
     """)
 
+Options = List(Either(String, Tuple(AnyRef, String)))
+OptionsGroups = Dict(String, Options)
+
+NotSelected = "" # TODO symbol
 
 class Select(InputWidget):
     ''' Single-select widget.
@@ -427,22 +470,25 @@ class Select(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
-    options = Either(List(Either(String, Tuple(String, String))),
-        Dict(String, List(Either(String, Tuple(String, String)))), help="""
-    Available selection options. Options may be provided either as a list of
-    possible string values, or as a list of tuples, each of the form
-    ``(value, label)``. In the latter case, the visible widget text for each
-    value will be corresponding given label. Option groupings can be provided
-    by supplying a dictionary object whose values are in the aforementioned
-    list format
-    """).accepts(List(Either(Null, String)), lambda v: [ "" if item is None else item for item in v ])
+    options = Either(Options, OptionsGroups, help="""
+    Available selection options.
 
-    value = String(default="", help="""
+    Options may be provided either as a list of possible string values, which
+    also act as options' labels, or as a list of tuples, each of the form
+    ``(value, label)``, where ``value`` can be of any type, not necessarily
+    a string. In the latter case, the visible widget text for each value will
+    be corresponding given label.
+
+    Option groupings can be provided by supplying a dictionary object whose
+    values are in the aforementioned list format.
+    """).accepts(List(Either(Null, String)), lambda v: [ NotSelected if item is None else item for item in v ])
+
+    value = AnyRef(default=NotSelected, help="""
     Initial or selected value.
-    """).accepts(Null, lambda _: "")
+    """).accepts(Null, lambda _: NotSelected)
 
 class MultiSelect(InputWidget):
     ''' Multi-select widget.
@@ -450,7 +496,7 @@ class MultiSelect(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     options = List(Either(String, Tuple(String, String)), help="""
@@ -477,7 +523,7 @@ class MultiChoice(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     options = List(Either(String, Tuple(String, String)), help="""
@@ -521,37 +567,56 @@ class ColorPicker(InputWidget):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     color = ColorHex(default='#000000', help="""
     The initial color of the picked color (named or hexadecimal)
     """)
 
-class ColorMap(InputWidget):
-    ''' Color map picker widget.
+class PaletteSelect(InputWidget):
+    ''' Color palette select widget.
 
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     value = Required(String, help="""
+    The name of the initial or selected color palette.
     """)
 
     items = Required(Seq(Tuple(String, Seq(Color))), help="""
+    A selection of named color palettes to choose from.
     """)
 
     swatch_width = NonNegative(Int, default=100, help="""
+    The width of the UI element showing the preview of a palette, in pixels.
     """)
 
-    swatch_height = NonNegative(Int, default=20, help="""
+    swatch_height = Either(Auto, NonNegative(Int), default="auto", help="""
+    The height of the UI element showing the preview of a palette, either in
+    pixels or automatically adjusted.
     """)
 
     ncols = Positive(Int, default=1, help="""
+    The number of columns to split the display of the palettes into.
     """)
 
+#-----------------------------------------------------------------------------
+# Legacy API
+#-----------------------------------------------------------------------------
+
+def ColorMap(*args: Any, **kwargs: Any) -> PaletteSelect:
+    ''' Color palette select widget.
+
+    .. deprecated:: 3.4.0
+        Use ``PaletteSelect`` widget instead.
+    '''
+    deprecated((3, 4, 0), "ColorMap widget", "PaletteSelect widget")
+    return PaletteSelect(*args, **kwargs)
+
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
